{{ DRV8711.spin }}

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  propTX  = 30 'programming output
  propRX  = 31 'programming input
  baudrate = 115_200
  MS = 80_000

  ' Pin definitions for testing
  SPI_MISO = 12
  SPI_MOSI = 13
  SPI_SCK  = 14
  DRV_CS   = 15 ' active high, remember

  DIRpin   = 16
  STEPpin  = 17
  RESETpin = 18 ' low to operate, high to reset
  SLEEPpin = 19 ' high to operate, low to sleep


  CTRL_REG   = 0

  DRV8711CTL_DEADTIME_400ns = $000
  DRV8711CTL_DEADTIME_450ns = $400
  DRV8711CTL_DEADTIME_650ns = $800
  DRV8711CTL_DEADTIME_850ns = $C00

  DRV8711CTL_IGAIN_5        = $000
  DRV8711CTL_IGAIN_10       = $100
  DRV8711CTL_IGAIN_20       = $200
  DRV8711CTL_IGAIN_40       = $300

  DRV8711CTL_STALL_INTERNAL = $000
  DRV8711CTL_STALL_EXTERNAL = $080

  DRV8711CTL_STEPMODE_MASK  = $078

  DRV8711CTL_FORCESTEP      = $004
  DRV8711CTL_REV_DIRECTION  = $002
  DRV8711CTL_ENABLE         = $001

  TORQUE_REG = 1

  DRV8711TRQ_BEMF_50us      = $000
  DRV8711TRQ_BEMF_100us     = $100
  DRV8711TRQ_BEMF_200us     = $200
  DRV8711TRQ_BEMF_300us     = $300
  DRV8711TRQ_BEMF_400us     = $400
  DRV8711TRQ_BEMF_600us     = $500
  DRV8711TRQ_BEMF_800us     = $600
  DRV8711TRQ_BEMF_1ms       = $700

  DRV8711TRQ_TORQUE_MASK    = $0FF

  OFF_REG    = 2

  DRV8711OFF_STEPMOTOR      = $000
  DRV8711OFF_DUALMOTORS     = $100

  DRV8711OFF_OFFTIME_MASK   = $0FF

  BLANK_REG  = 3

  DRV8711BLNK_ADAPTIVE_BLANK = $100
  DRV8711BLNK_BLANKTIME_MASK = $0FF

  DECAY_REG  = 4

  DRV8711DEC_SLOW_DECAY     = $000
  DRV8711DEC_SLOW_MIXED     = $100
  DRV8711DEC_FAST_DECAY     = $200
  DRV8711DEC_MIXED_DECAY    = $300
  DRV8711DEC_SLOW_AUTOMIX   = $400
  DRV8711DEC_AUTOMIX        = $500
  DRV8711DEC_DECAYTIME_MASK = $0FF

  STALL_REG  = 5

  DRV8711STL_DIVIDE_32      = $000
  DRV8711STL_DIVIDE_16      = $400
  DRV8711STL_DIVIDE_8       = $800
  DRV8711STL_DIVIDE_4       = $C00
  DRV8711STL_STEPS_1        = $000
  DRV8711STL_STEPS_2        = $100
  DRV8711STL_STEPS_4        = $200
  DRV8711STL_STEPS_8        = $300
  DRV8711STL_THRES_MASK     = $0FF

  DRIVE_REG  = 6

  DRV8711DRV_HIGH_50mA      = $000
  DRV8711DRV_HIGH_100mA     = $400
  DRV8711DRV_HIGH_150mA     = $800
  DRV8711DRV_HIGH_200mA     = $C00

  DRV8711DRV_LOW_100mA      = $000
  DRV8711DRV_LOW_200mA      = $100
  DRV8711DRV_LOW_300mA      = $200
  DRV8711DRV_LOW_400mA      = $300

  DRV8711DRV_HIGH_250ns     = $000
  DRV8711DRV_HIGH_500ns     = $040
  DRV8711DRV_HIGH_1us       = $080
  DRV8711DRV_HIGH_2us       = $0C0

  DRV8711DRV_LOW_250ns      = $000
  DRV8711DRV_LOW_500ns      = $010
  DRV8711DRV_LOW_1us        = $020
  DRV8711DRV_LOW_2us        = $030

  DRV8711DRV_OCP_1us        = $000
  DRV8711DRV_OCP_2us        = $004
  DRV8711DRV_OCP_4us        = $008
  DRV8711DRV_OCP_8us        = $00C

  DRV8711DRV_OCP_250mV      = $000
  DRV8711DRV_OCP_500mV      = $001
  DRV8711DRV_OCP_750mV      = $002
  DRV8711DRV_OCP_1000mV     = $003

  STATUS_REG = 7

  DRV8711STS_LATCHED_STALL  = $080
  DRV8711STS_STALL          = $040
  DRV8711STS_PREDRIVE_B     = $020
  DRV8711STS_PREDRIVE_A     = $010
  DRV8711STS_UNDERVOLT      = $008
  DRV8711STS_OVERCUR_B      = $004
  DRV8711STS_OVERCUR_A      = $002
  DRV8711STS_OVERTEMP       = $001


OBJ
  Clock : "Clock"
  debug: "SerialMirror"  ' debug port
  fmt: "Format"          ' Format object
  spi: "DRV8711_SPI"

VAR
  byte  buffer[100] 'buffer to assemble output strings
  long  lock
  long  drive_level


PUB Main | diff
  Clock.init (5_000_000)
  Clock.setclock (XTAL1 + PLL16X) 'Set clock to really fast
  debug.start (propRX, propTX, 0, baudrate)
  fmt.sprintf (@buffer, string("ts = %i ."), cnt)
  debug.str (@buffer)
  debug.tx (10)

  lock := locknew
  fmt.sprintf (@buffer, string("SPI lock claimed is %i"), lock)
  debug.str (@buffer)
  debug.tx (10)
  if lock < 0
    fmt.sprintf (@buffer, string ("LOCKS ALL USED!!"), 0)
    debug.str (@buffer)
    debug.tx (10)

  fmt.sprintf (@buffer, string ("cog = %i ."), spi.Start (SPI_SCK, SPI_MISO, SPI_MOSI))
  debug.str (@buffer)
  debug.tx (10)

  DIRA |= (1 << RESETpin) | (1 << SLEEPpin) | (1 << DRV_CS)
  DIRA |= (1 << STEPpin) | (1 << DIRpin)
  OUTA |= (1 << SLEEPpin)  ' wake up
  waitcnt (CNT + MS)
  OUTA |= (1 << RESETpin)  ' reset
  waitcnt (CNT + MS)
  OUTA &= !(1 << RESETpin) ' end reset
  waitcnt (CNT + MS)

  fmt.sprintf (@buffer, string ("reset"), 0)
  debug.str (@buffer)
  debug.tx (10)

  show_registers
  setup_DRV8711 (60, 16, 2, 0, 1, 0)
  show_registers
  {
  repeat

    OUTA |= |< STEPpin
    OUTA &= ! (|< STEPpin | |< DIRpin)
    OUTA ^= ((CNT >> 23) & 1)<<DIRpin

    waitcnt (CNT + 9000)
  }
  FRQA := 0
  diff := 1000
  CTRA := (%00100_000 << 23) | STEPpin
  repeat
    FRQA += diff
    if FRQA > 3000000 or FRQA < -3000000
      diff := -diff


PUB show_registers | reg, regval
  reg := 0
  repeat
    regval := spi.read12BitReg (DRV_CS, 8|reg)
    fmt.sprintf (@buffer, string ("%i "), reg)
    debug.str (@buffer)
    fmt.sprintf (@buffer, string ("$%x"), regval)
    debug.str (@buffer)
    debug.tx (10)
    reg += 1
  until reg > 7


PUB setup_DRV8711 (drive, microsteps, decay_mode, gate_speed, gate_drive, deadtime) | microstep_code, decay_time, control_reg
  drive_level := drive
  microstep_code := 0
  repeat until microsteps < 2
    microsteps >>= 1
    microstep_code += 1
  decay_time := $20

  ' CONTROL REG    ddggsmmmmSDE
  ' dd = deadtime (400, 450, 650, 850)ns
  ' gg = Igain (5, 10, 20, 40)   note threshold after gain = 2.75V, so (550mV, 275mV, 138mV, 69mV)
  ' s = (internal, external) stall detect
  ' mmmm = (full, half, 1/4, 1/8, 1/16, 1/32, 1/64, 1/128, 1/256,...) microstepping mode
  ' S = (nop, forcestep)  - self-clears
  ' D = (normal, reverse) XORed with direction pin
  ' E = (off, on)  enable motor(s)
  control_reg := ((deadtime & 3) << 10) | ((microstep_code & $F) << 3) | DRV8711CTL_IGAIN_10 | DRV8711CTL_STALL_INTERNAL

  spi.write12BitReg (DRV_CS, CTRL_REG, control_reg)

  ' TORQUE REG  -ssstttttttt
  ' sss = (50,100,200,300,400,600,800,1000)us, backEMF sample threshhold
  ' tttttttt = torque level
  spi.write12BitReg (DRV_CS, TORQUE_REG, DRV8711TRQ_BEMF_50us | (drive & DRV8711TRQ_TORQUE_MASK))

  ' OFF REG  ---moooooooo
  ' m = (stepper, DCmotors)
  ' oooooooo = off time in 500ns units (500ns..128us)
  spi.write12BitReg (DRV_CS, OFF_REG, DRV8711OFF_STEPMOTOR | ($030 & DRV8711OFF_OFFTIME_MASK))

  ' BLANK REG  ---abbbbbbbb
  ' a = (off,on)  adaptive blanking time
  ' bbbbbbbb = blank time in units 20ns, but 1us minimum, so 1us--5.12us
  spi.write12BitReg (DRV_CS, BLANK_REG, DRV8711BLNK_ADAPTIVE_BLANK | ($80 & DRV8711BLNK_BLANKTIME_MASK))

  ' DECAY REG  -mmmdddddddd
  ' mmm = (slow, slow/mix, fast, mixed, slow/auto, auto, ...)
  ' dddddddd = mixed decay time in units 0.5us--128us
  spi.write12BitReg (DRV_CS, DECAY_REG, (decay_mode & 7) << 8 | (decay_time & DRV8711DEC_DECAYTIME_MASK))

  ' STALL REG  ddsstttttttt
  ' dd = (1/32, 1/16, 1/8, 1/4)  divide backEMF
  ' ss = (1, 2, 4, 8) steps before stall asserted
  ' tttttttt = stall threshold
  spi.write12BitReg (DRV_CS, STALL_REG, DRV8711STL_DIVIDE_8 | DRV8711STL_STEPS_1 | ($20 & DRV8711STL_THRES_MASK))

  ' DRIVE REG  HHLLhhllddoo
  ' HH = (50, 100, 150, 200)mA  high gate drive level
  ' LL = (100, 200, 300, 400)mA low gate drive level
  ' hh = (250, 500, 1000, 2000)ns high gate active drive time
  ' ll = (250, 500, 1000, 2000)ns low gate active drive time
  ' dd = (1, 2, 4, 8)us  OCP deglitch time
  ' oo = (250, 500, 750, 1000)mV  OCP threshold (across FET)
  spi.write12BitReg (DRV_CS, DRIVE_REG,  ((gate_drive & 3) * $500) | ((gate_speed & 3) * $050) | DRV8711DRV_OCP_1us | DRV8711DRV_OCP_250mV)

  ' STATUS REG  ----LSBAUbaT
  ' L = latched stall fault (write 0 to clear)
  ' S = transient stall fault (self clearing)
  ' B = predriver B fault (write 0 to clear)
  ' A = predriver A fault (write 0 to clear)
  ' U = undervoltage fault (self clearing)
  ' b = B overcurrent fault (write 0 to clear)
  ' a = A overcurrent fault (write 0 to clear)
  ' T = overtemperature fault (self clearing)
  spi.write12BitReg (DRV_CS, STATUS_REG, 0)  ' clear all status bits

  spi.write12BitReg (DRV_CS, CTRL_REG, control_reg | DRV8711CTL_ENABLE)



